<?php 

namespace PortalPeople\magento\controllers;

use yii\web\Controller;
use Yii;

class CartsController extends Controller
{
    public function actionIndex()
    {
        $searchModel = new \backend\models\Cartsbesearch();
        $searchModel->account_code = 'GAR001';
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
        return $this->render('@backend/views/cart/index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }
}